@@ -15,13 +15,17 @@ import shutil
 from typing import Dict, List, NamedTuple, Union
 from pathlib import Path
 
+from vantage6.common.docker.addons import get_container, running_in_docker
 from vantage6.common.globals import APPNAME
 from vantage6.node.docker.docker_base import DockerBaseManager
-from vantage6.node.docker.utils import running_in_docker
 from vantage6.node.docker.vpn_manager import VPNManager
 from vantage6.node.util import logger_name
-from vantage6.node.docker.network_manager import IsolatedNetworkManager
-from vantage6.node.docker.task_manager import DockerTaskManager
+from vantage6.common.docker.network_manager import NetworkManager
+from vantage6.node.docker.task_manager import DockerTaskManager, TaskStatus
+from vantage6.node.docker.exceptions import (
+    UnknownAlgorithmStartFail,
+    PermanentAlgorithmStartFail
+)
 
 log = logging.getLogger(logger_name(__name__))
 
@@ -45,7 +49,7 @@ class DockerManager(DockerBaseManager):
     """
     log = logging.getLogger(logger_name(__name__))
 
-    def __init__(self, ctx, isolated_network_mgr: IsolatedNetworkManager,
+    def __init__(self, ctx, isolated_network_mgr: NetworkManager,
                  vpn_manager: VPNManager, tasks_dir: Path) -> None:
         """ Initialization of DockerManager creates docker connection and
             sets some default values.
@@ -54,7 +58,7 @@ class DockerManager(DockerBaseManager):
             ----------
             ctx: DockerNodeContext or NodeContext
                 Context object from which some settings are obtained
-            isolated_network_mgr: IsolatedNetworkManager
+            isolated_network_mgr: NetworkManager
                 Manager for the isolated network
             vpn_manager: VPNManager
                 VPN Manager object
@@ -69,10 +73,14 @@ class DockerManager(DockerBaseManager):
         self.algorithm_env = config.get('algorithm_env', {})
         self.vpn_manager = vpn_manager
         self.__tasks_dir = tasks_dir
+        self.alpine_image = config.get('alpine')
 
         # keep track of the running containers
         self.active_tasks: List[DockerTaskManager] = []
 
+        # keep track of the containers that have failed to start
+        self.failed_tasks: List[DockerTaskManager] = []
+
         # before a task is executed it gets exposed to these regex
         self._allowed_images = config.get("allowed_images")
 
@@ -91,6 +99,9 @@ class DockerManager(DockerBaseManager):
         # set database uri and whether or not it is a file
         self._set_database(config)
 
+        # keep track of linked docker services
+        self.linked_services: List[str] = []
+
     def _set_database(self, config: Dict) -> None:
         """"
         Set database location and whether or not it is a file
@@ -220,7 +231,9 @@ class DockerManager(DockerBaseManager):
         while self.active_tasks:
             task = self.active_tasks.pop()
             task.cleanup()
-        self.isolated_network_mgr.cleanup()
+        for service in self.linked_services:
+            self.isolated_network_mgr.disconnect(service)
+        self.isolated_network_mgr.delete(kill_containers=True)
 
     def run(self, result_id: int,  image: str, docker_input: bytes,
             tmp_vol_name: str, token: str, database: str
@@ -270,18 +283,42 @@ class DockerManager(DockerBaseManager):
             tasks_dir=self.__tasks_dir,
             isolated_network_mgr=self.isolated_network_mgr,
             databases=self.databases,
-            docker_volume_name=self.data_volume_name
-        )
-        database = database if len(database) else 'default'
-        vpn_ports = task.run(
-            docker_input=docker_input, tmp_vol_name=tmp_vol_name, token=token,
-            algorithm_env=self.algorithm_env, database=database
+            docker_volume_name=self.data_volume_name,
+            alpine_image=self.alpine_image
         )
+        database = database if (database and len(database)) else 'default'
+
+
+        # attempt to kick of the task. If it fails do to unknown reasons we try
+        # again. If it fails permanently we add it to the failed tasks to be
+        # handled by the speaking worker of the node
+        attempts = 1
+        while not (task.status == TaskStatus.STARTED) and attempts < 3:
+            try:
+                vpn_ports = task.run(
+                    docker_input=docker_input, tmp_vol_name=tmp_vol_name,
+                    token=token, algorithm_env=self.algorithm_env,
+                    database=database
+                )
+
+            except UnknownAlgorithmStartFail:
+                self.log.exception(f'Failed to start result {result_id} due '
+                                   'to unknown reason. Retrying')
+                time.sleep(1) # add some time before retrying the next attempt
+
+            except PermanentAlgorithmStartFail:
+                break
+
+            attempts += 1
 
         # keep track of the active container
-        self.active_tasks.append(task)
-
-        return vpn_ports
+        if task.status == TaskStatus.FAILED \
+                or task.status == TaskStatus.PERMANENTLY_FAILED:
+            self.failed_tasks.append(task)
+            return None
+        else:
+            self.active_tasks.append(task)
+            return vpn_ports
 
     def get_result(self) -> Result:
         """
@@ -300,25 +337,34 @@ class DockerManager(DockerBaseManager):
         # get finished results and get the first one, if no result is available
         # this is blocking
         finished_tasks = []
-        while not finished_tasks:
+        while (not finished_tasks) and (not self.failed_tasks):
             finished_tasks = [t for t in self.active_tasks if t.is_finished()]
             time.sleep(1)
 
-        # at least one task is finished
-        finished_task = finished_tasks.pop()
-        self.log.debug(f"Result id={finished_task.result_id} is finished")
+        if finished_tasks:
+            # at least one task is finished
 
-        # Check exit status and report
-        logs = finished_task.report_status()
+            finished_task = finished_tasks.pop()
+            self.log.debug(f"Result id={finished_task.result_id} is finished")
 
-        # Cleanup containers
-        finished_task.cleanup()
+            # Check exit status and report
+            logs = finished_task.report_status()
 
-        # Retrieve results from file
-        results = finished_task.get_results()
+            # Cleanup containers
+            finished_task.cleanup()
 
-        # remove finished tasks from active task list
-        self.active_tasks.remove(finished_task)
+            # Retrieve results from file
+            results = finished_task.get_results()
+
+            # remove finished tasks from active task list
+            self.active_tasks.remove(finished_task)
+
+        else:
+            # at least one task failed to start
+            finished_task = self.failed_tasks.pop()
+            finished_task.status_code = 9
+            logs = 'Container failed to start'
+            results = b''
 
         return Result(
             result_id=finished_task.result_id,
@@ -347,3 +393,30 @@ class DockerManager(DockerBaseManager):
             except docker.errors.APIError as e:
                 self.log.warn(f"Could not login to {registry.get('registry')}")
                 self.log.debug(e)
+
+    def link_container_to_network(self, container_name: str,
+                                  config_alias: str) -> None:
+        """
+        Link a docker container to the isolated docker network
+
+        Parameters
+        ----------
+        container_name: str
+            Name of the docker container to be linked to the network
+        config_alias: str
+            Alias of the docker container defined in the config file
+        """
+        container = get_container(
+            docker_client=self.docker, name=container_name
+        )
+        if not container:
+            self.log.error(f"Could not link docker container {container_name} "
+                           "that was specified in the configuration file to "
+                           "the isolated docker network.")
+            self.log.error("Container not found!")
+            return
+        self.isolated_network_mgr.connect(
+            container_name=container_name,
+            aliases=[config_alias]
+        )
+        self.linked_services.append(container_name)
